#
#    forest/privs.py implements privileges for the forest message board
#    Copyright (C) 2002  Michael Urman
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
privs.py

describes privileges a user has on a global and per-tree basis
"""

class priv:

    privlist = 'read write respond'.split()

    def __init__(self):
        """__init__(self) -> priv

        initialize a priv structure to an initial empty set"""

        for attr in priv.privlist:
            setattr(self, attr, 0)

    def __and__(s, o):
        """priv & priv -> priv

        Return the logical and of two priv objects"""

        result = priv()
        for attr in priv.privlist:
            setattr(result, attr, getattr(s, attr) & getattr(o, attr))
        return result

class privs:

    """privs stores the privileges a user has, globally, local to a tree, and
    the logical and of the two.  These are stored in self.forest,
    self.tree[tree], and self.both[tree] respectively once they have been
    loaded.
    
    To handle some format of privilege flushing, it is expected that the server
    will delete relevent privileges as necessary, forcing an automatic
    reload."""

    def __init__(self, user):
        """__init__(self, user) -> privs

        create a privilege object with global permissions"""

        self.user = user
        self.tree = {}
        self.both = {}

    def get_priv(self, tree=None):
        """get_priv(self, tree=None) -> priv

        Return (global & tree) priv object.  Load as necessary."""

        try:
            forest_priv = self.forest
        except AttributeError:
            forest_priv = self.forest = self.lookup_forest()
            if self.tree.has_key(tree):
                del self.tree[tree]
            if self.both.has_key(tree):
                del self.both[tree]

        if tree is None:
            return forest_priv

        try:
            both_priv = self.both[tree]
        except AttributeError:
            try:
                tree_priv = self.tree[tree]
            except AttributeError:
                self.tree[tree] = self.lookup_tree(tree)
                tree_priv = self.tree[tree]
            both_priv = forest_priv & tree_priv
            self.both[tree] = both_priv

        return both_priv
